#include <SPI.h>
#include <EEPROMex.h>
#include <TinyGPS++.h>
#include <SoftwareSerial.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
#include <Wire.h>
#include <LSM303.h>

/*********************************************************************
	MTSuzuki Car Tracker with Compass
	When set button is pushed, GPS stores current location in EEPROM
	When the user moves away from set point, distance and direction 
	are calculated using the stored location and the current location, 
	The compass is used to orient the display so that it points back
	to the set point and displays the distance. 
	TinyGPS++ is used for the GPS and (TinyGPSPlus) object is
	used to find distance and direction 
	Uses LM303dhlc compass, Ublox-NEO-6M GPS, LCD5110 display
*********************************************************************/

/*********************************************************************
This is an example sketch for our Monochrome Nokia 5110 LCD Displays

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/products/338

These displays use SPI to communicate, 4 or 5 pins are required to
interface

Adafruit invests time and resources providing this open source code,
please support Adafruit and open-source hardware by purchasing
products from Adafruit!

Written by Limor Fried/Ladyada  for Adafruit Industries.
BSD license, check license.txt for more information
All text above, and the splash screen must be included in any redistribution
*********************************************************************/

// lsm303 compass stuff
LSM303 compass;
int heading;
double distancem;	// distance in meters
double courseTo;	// heading 

byte setPin = 17;		//pushbutton, D17 is A3

// LCD 5110 connections
// D4 - Serial clock out (CLK)
// D14 - Serial data out (DIN)
// D2 - Data/Command select (D/C)
// D15 - LCD chip select (CE)
// D16 - LCD reset (RST)
Adafruit_PCD8544 display = Adafruit_PCD8544(4, 14, 2, 15, 16);

// GPS connections (RX pin connects to TX on module and v.v.) and Baud rate
static const int RXPin = 5, TXPin = 6;
static const uint32_t GPSBaud = 9600;

// The TinyGPS++ object
TinyGPSPlus gps;

// GPS Latitude[0]/Longitude[1]
double setLL[2];	//stored location
int addressLat = 0;	// EEPROM address 
int addressLng = 4;	// EEPROM address, double is 4 bytes
double myLL[2];		//current GPS location

// The serial connection to the GPS device
SoftwareSerial sGPS(RXPin, TXPin);

// The following are for displaying direction on LCD5110
//position 0-N, 1-NNE, 2-NE, 3-ENE, 4-E, 5-ESE, 6-SE, 7-SSE
//         8-S, 9-SSW, 10-SW, 11-WSW, 12-W, 13-WNW, 14-NW, 15-NNW
byte position;

//x, y coordinates for pointer head on LCD5110
byte cursorxy[16][2] = {
    // 0-North 1-NNE   2-NE   3-ENE	 4-East  5-ESE   6-SE    7-SSE
       39, 0,  59, 0,  79, 0, 79, 8, 79, 16, 79, 24, 79, 32, 59, 32,
    // 8-South 9-SSW   10-SW  11-WSW 12-West 13-WNW 14-NW 15-NNW
       39, 32, 29, 32, 0, 32, 0, 24, 0, 16,  0, 8,  0, 0, 29, 0 };

// Pointer character					  
char LCDchar[16] = {'A','A','A','>','>','>','X','V','V','V','X','L','<','<','<','A'}; 					   
//x1, y1 ,x2, y2 coordinates for line
byte dline[16][4] = {
    // 0-North          1-NNE             2-NE			  3-ENE
       41, 7, 41, 39,   30, 39, 61, 5,    80, 7, 0, 39,   0, 24, 80, 11,
    // 4-E,              5-ESE            6-SE,           7-SSE
       0, 19, 80, 19,    0, 10, 80, 27,   0, 0, 79, 31,   31, 0, 61, 31,
    // 8-S			     9-SSW			  10-SW			  11-WSW
       41, 0, 41, 31,    61, 0, 31, 31,   5, 31, 83, 0,   83, 10, 2, 26, 
    // 12-W			     13-WNW			  14-NW			  15-NNW	
       0, 19, 83, 19,    83, 26, 4, 10,   0, 0, 83, 39,   60, 32, 31, 0};

//Prototypes
void setSetPoint();			// set SetPoint
void getGPS();				// get current GPS data
void calculate();			// calculate distance and heading
int getHeading();			// get heading using compass
byte getPostion(int);		// get position using heading
void displayDirection();	// display info 

void setup(){
	pinMode(setPin, INPUT_PULLUP);
	Serial.begin(115200);
	
	// start GPS
	sGPS.begin(GPSBaud);

	// setup for compass
	Wire.begin();
	compass.init();
	compass.enableDefault();
	// Values from Calibrate sketch
	compass.m_min = (LSM303::vector<int16_t>){-433, -600, -546};
	compass.m_max = (LSM303::vector<int16_t>){+570, +488, +579};
	
	//Read in Set Point from EEPROM
	setLL[0] = EEPROM.readDouble(addressLat);
	setLL[1] = EEPROM.readDouble(addressLng);
	
	//LCD5110 setup
	display.begin();
	display.setContrast(65);	// Choose best contrast
	display.display(); 			// show splashscreen
	delay(20);
	display.clearDisplay();   	// clears the screen and buffer
}

void loop(){
	// wait until GPS is working
	while (sGPS.available() > 0)
		if (gps.encode(sGPS.read())){
			if(digitalRead(setPin)==LOW){	// Change set point to current location
				setSetPoint();
			}
			else{
				getGPS();			// get location
				calculate();		// calculate distance and direction
				// calculate position by compass heading and courseTo (GPS)
				position = getPostion(getHeading() + courseTo );
				displayDirection();	// display direction and distance
			}
		}
}

// set SetPoint
void setSetPoint(){
	setLL[0]= gps.location.lat();
	setLL[1]= gps.location.lng();
	EEPROM.updateDouble(addressLat, setLL[0]);	// write to EEPROM
	EEPROM.updateDouble(addressLng, setLL[1]);	// write to EEPROM
	display.clearDisplay();   	// clears the screen and buffer
	display.println("Set Point:");
	display.println("Latitude:");
	display.println(setLL[0],6);
	display.println("Longitude:");
	display.println(setLL[1],6);
	display.display(); 			// show screen
	delay(2000);
}	

//get current GPS data
void getGPS(){
	if (gps.location.isValid()){
		myLL[0] = gps.location.lat();
		myLL[1] = gps.location.lng();
	}
}

//get heading using compass
int getHeading(){
	compass.read();
	heading = (int)compass.heading();
	return ((630 - heading) % 360);	// The lm303 is mounted 270 degrees off
}

//calculate distance and heading
void calculate(){
	if ((setLL[0] !=0.0) && (setLL[1] !=0.0)){
		distancem = TinyGPSPlus::distanceBetween(myLL[0],myLL[1],setLL[0],setLL[1]);
		courseTo =  TinyGPSPlus::courseTo(myLL[0],myLL[1],setLL[0],setLL[1]);
	}
}

//get position using heading and courseTo
byte getPostion(int headplus){

	headplus = headplus % 360;
	if ((headplus > 348) || (headplus < 12)) return 0;
	if ((headplus > 11) && (headplus < 34)) return 1;
	if ((headplus > 33) && (headplus < 57)) return 2;
	if ((headplus > 56) && (headplus < 79)) return 3;
	if ((headplus > 78) && (headplus < 102)) return 4;
	if ((headplus > 101) && (headplus < 124)) return 5;
	if ((headplus > 123) && (headplus < 147)) return 6;
	if ((headplus > 146) && (headplus < 169)) return 7;
	if ((headplus > 168) && (headplus < 192)) return 8;
	if ((headplus > 191) && (headplus < 214)) return 9;
	if ((headplus > 213) && (headplus < 237)) return 10;
	if ((headplus > 236) && (headplus < 259)) return 11;
	if ((headplus > 258) && (headplus < 282)) return 12;
	if ((headplus > 281) && (headplus < 304)) return 13;
	if ((headplus > 303) && (headplus < 327)) return 14;
	if ((headplus > 326) && (headplus < 349)) return 15;
} 

// display info 
void displayDirection(){
	int feet;
	display.clearDisplay();
	display.setCursor(cursorxy[position][0],cursorxy[position][1]); // head of arrow
	display.print(LCDchar[position]);
	display.drawLine(dline[position][0], dline[position][1], dline[position][2], dline[position][3], BLACK);
	display.setCursor(0,40); 
	feet = (int)(distancem*3.2808399);	// convert meters to feet
	if (feet >= 5280){
		display.print("miles: ");
		display.print(feet/5280.0);
	}else{
		display.print("Feet: ");
		display.print(feet);
	}
	display.display();
	delay(1000);
}
